@@ -728,11 +728,9 @@ giraffe_fov_build(GiFieldOfView* result, GiRebinning* rebinning,
                         cxdouble yorigin = giraffe_cube_get_height(_scube) / 2.;
 
                         cxdouble xvalue =
-                                cpl_propertylist_get_double(properties,
-                                                            GIALIAS_RADEG);
+                                cpl_table_get_double(ssf, "RA", 0, NULL);
                         cxdouble yvalue =
-                                cpl_propertylist_get_double(properties,
-                                                            GIALIAS_DECDEG);
+                                cpl_table_get_double(ssf, "DEC", 0, NULL);
                         cxdouble orientation =
                                 cpl_table_get_double(ssf, "ORIENT", 0, NULL);
 
