@@ -1,7 +1,7 @@
 /* $Id$
  *
  * This file is part of the GIRAFFE Pipeline
- * Copyright (C) 2002-2006 European Southern Observatory
+ * Copyright (C) 2002-2014 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <cxstring.h>
 
 #include <cpl_error.h>
+#include <cpl_array.h>
 
 #include "gialias.h"
 #include "gitable.h"
@@ -269,13 +270,12 @@ giraffe_table_copy_matrix(GiTable *table, const cxchar *name,
 
     const cxchar *fctid = "giraffe_table_copy_matrix";
 
-    const cxchar *label = NULL;
-
     cxint nrow = 0;
     cxint ncol = 0;
     cxint count = 0;
 
-    cpl_table *_table;
+    cpl_array *labels = NULL;
+    cpl_table *_table = NULL;
 
 
     cx_assert(table != NULL);
@@ -290,21 +290,29 @@ giraffe_table_copy_matrix(GiTable *table, const cxchar *name,
 
     _table = giraffe_table_get(table);
 
-    label = cpl_table_get_column_name(_table);
+    labels = cpl_table_get_column_names(_table);
+    cx_assert(cpl_array_get_size(labels) > 0);
+
 
     if (name != NULL) {
 
         if (cpl_table_has_column(_table, name) == 0) {
+
+            cpl_array_delete(labels);
+            cpl_error_set(fctid, CPL_ERROR_DATA_NOT_FOUND);
+
             return 1;
+
         }
         else {
 
             /*
-             * Skip to the first target column
+             * Move to the first target column. No check whether count goes
+             * out of range is needed here since it is already known that
+             * a column with name 'name' exists.
              */
 
-            while (label && strcmp(label, name) != 0) {
-                label = cpl_table_get_column_name(NULL);
+            while (strcmp(cpl_array_get_string(labels, count), name) != 0) {
                 ++count;
             }
 
@@ -312,9 +320,15 @@ giraffe_table_copy_matrix(GiTable *table, const cxchar *name,
 
     }
 
+
     if (cpl_table_get_nrow(_table) != nrow ||
         cpl_table_get_ncol(_table) - count < ncol) {
+
+        cpl_array_delete(labels);
+        cpl_error_set(fctid, CPL_ERROR_INCOMPATIBLE_INPUT);
+
         return 1;
+
     }
     else {
 
@@ -323,71 +337,64 @@ giraffe_table_copy_matrix(GiTable *table, const cxchar *name,
         cxdouble *elements = cpl_matrix_get_data(matrix);
 
 
-        if (!label) {
-            cpl_error_set(fctid, CPL_ERROR_ACCESS_OUT_OF_RANGE);
-            return 1;
-        }
+        for (i = 0; i < ncol; ++i) {
 
-        for (i = 0; i < ncol; i++) {
+            const cxchar *label = cpl_array_get_string(labels, count + i);
 
             cpl_type type = cpl_table_get_column_type(_table, label);
 
 
             switch (type) {
-            case CPL_TYPE_INT:
-            {
-                cxint j;
+                case CPL_TYPE_INT:
+                {
+                    register cxint j;
 
-                for (j = 0; j < nrow; j++) {
-                    cpl_table_set_int(_table, label, j,
-                                      (cxint) elements[j * ncol + i]);
+                    for (j = 0; j < nrow; ++j) {
+                        cpl_table_set_int(_table, label, j,
+                                          (cxint) elements[j * ncol + i]);
+                    }
+                    break;
                 }
-                break;
-            }
 
-            case CPL_TYPE_FLOAT:
-            {
-                cxint j;
+                case CPL_TYPE_FLOAT:
+                {
+                    register cxint j;
 
-                for (j = 0; j < nrow; j++) {
-                    cpl_table_set_float(_table, label, j,
-                                        (cxfloat) elements[j * ncol + i]);
+                    for (j = 0; j < nrow; ++j) {
+                        cpl_table_set_float(_table, label, j,
+                                            (cxfloat) elements[j * ncol + i]);
+                    }
+                    break;
                 }
-                break;
-            }
 
-            case CPL_TYPE_DOUBLE:
-            {
-                cxint j;
+                case CPL_TYPE_DOUBLE:
+                {
+                    register cxint j;
 
-                for (j = 0; j < nrow; j++) {
-                    cpl_table_set_double(_table, label, j,
-                                         elements[j * ncol + i]);
+                    for (j = 0; j < nrow; ++j) {
+                        cpl_table_set_double(_table, label, j,
+                                             elements[j * ncol + i]);
+                    }
+                    break;
                 }
-                break;
-            }
 
-            case CPL_TYPE_STRING:
-            default:
-                cpl_error_set(fctid, CPL_ERROR_INVALID_TYPE);
-                return 1;
-                break;
-            }
+                case CPL_TYPE_STRING:
+                default:
+                {
+                    cpl_array_delete(labels);
+                    cpl_error_set(fctid, CPL_ERROR_INVALID_TYPE);
 
-            if (i < ncol - 1) {
-
-                label = cpl_table_get_column_name(NULL);
-
-                if (!label) {
-                    cpl_error_set(fctid, CPL_ERROR_ACCESS_OUT_OF_RANGE);
                     return 1;
+                    break;
                 }
-
             }
+
         }
 
     }
 
+    cpl_array_delete(labels);
+
     return 0;
 
 }
