@@ -62,7 +62,7 @@
 
 static const cxchar *_giraffe_license =
  " This file is part of the GIRAFFE Instrument Pipeline\n"
- " Copyright (C) 2002-2006 European Southern Observatory\n"
+ " Copyright (C) 2002-2014 European Southern Observatory\n"
  "\n"
  " This program is free software; you can redistribute it and/or modify\n"
  " it under the terms of the GNU General Public License as published by\n"
@@ -598,11 +598,12 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
 
         cx_string *key = cx_string_new();
 
-        const cpl_frame *frame = cpl_frameset_get_first_const(set);
+        const cpl_frame *frame = NULL;
 
-        while (frame != NULL) {
+        cpl_frameset_iterator *it = cpl_frameset_iterator_new(set);
 
-            cxint status = 0;
+
+        while ((frame = cpl_frameset_iterator_get_const(it)) != NULL) {
 
             cpl_frame_group group = cpl_frame_get_group(frame);
 
@@ -617,15 +618,17 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
             case CPL_FRAME_GROUP_RAW:
             {
                 ++nraw;
-                status = _giraffe_add_frame_info(plist, base, tag, sequence,
-                                                 nraw, 0);
+                cxint status = _giraffe_add_frame_info(plist, base, tag,
+                                                       sequence, nraw, 0);
 
                 if (status != 0) {
                     if (base != NULL) {
                         cx_free((cxchar *)base);
                     }
 
+                    cpl_frameset_iterator_delete(it);
                     cx_string_delete(key);
+
                     return -2;
                 }
 
@@ -639,15 +642,17 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
 
 
                 ++ncalib;
-                status = _giraffe_add_frame_info(plist, base, tag, sequence,
-                                                 ncalib, 1);
+                cxint status = _giraffe_add_frame_info(plist, base, tag,
+                                                       sequence, ncalib, 1);
 
                 if (status != 0) {
                     if (base != NULL) {
                         cx_free((cxchar *)base);
                     }
 
+                    cpl_frameset_iterator_delete(it);
                     cx_string_delete(key);
+
                     return -3;
                 }
 
@@ -658,7 +663,9 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
                         cx_free((cxchar *)base);
                     }
 
+                    cpl_frameset_iterator_delete(it);
                     cx_string_delete(key);
+
                     return -3;
                 }
 
@@ -690,7 +697,9 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
                                 cx_free((cxchar *)base);
                             }
 
+                            cpl_frameset_iterator_delete(it);
                             cx_string_delete(key);
+
                             return -3;
                         }
 
@@ -712,10 +721,11 @@ giraffe_add_frameset_info(cpl_propertylist *plist, const cpl_frameset *set,
                 cx_free((cxchar *)base);
             }
 
-            frame = cpl_frameset_get_next_const(set);
+            cpl_frameset_iterator_advance(it, 1);
 
         }
 
+        cpl_frameset_iterator_delete(it);
         cx_string_delete(key);
 
     }
@@ -1126,6 +1136,8 @@ giraffe_frameset_set_groups(cpl_frameset* set, GiGroupInfo *groups)
 
     cpl_frame* frame = NULL;
 
+    cpl_frameset_iterator *it = NULL;
+
 
     if (set == NULL) {
         return -1;
@@ -1135,9 +1147,9 @@ giraffe_frameset_set_groups(cpl_frameset* set, GiGroupInfo *groups)
         return 0;
     }
 
-    frame = cpl_frameset_get_first(set);
+    it = cpl_frameset_iterator_new(set);
 
-    while (frame != NULL) {
+    while ((frame = cpl_frameset_iterator_get(it)) != NULL) {
 
         const cxchar* tag = cpl_frame_get_tag(frame);
 
@@ -1156,10 +1168,12 @@ giraffe_frameset_set_groups(cpl_frameset* set, GiGroupInfo *groups)
 
         }
 
-        frame = cpl_frameset_get_next(set);
+        cpl_frameset_iterator_advance(it, 1);
 
     }
 
+    cpl_frameset_iterator_delete(it);
+
     return 0;
 
 }
